--[[	PhantomPlates Personal NamePlate Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

--------------------------
--[[	Constants	]]
--------------------------
local DefaultYPos=-128;
local PowerRestState={
--	<not defined> = Rest at max
	[Enum.PowerType.Rage]=0;
	[Enum.PowerType.ComboPoints]=0;
	[Enum.PowerType.RunicPower]=0;
	[Enum.PowerType.SoulShards]=3;
	[Enum.PowerType.LunarPower]=0;
	[Enum.PowerType.HolyPower]=0;
	[Enum.PowerType.Maelstrom]=0;
	[Enum.PowerType.Chi]=0;
	[Enum.PowerType.Insanity]=0;
	[Enum.PowerType.ArcaneCharges]=0;
	[Enum.PowerType.Fury]=0;
	[Enum.PowerType.Pain]=0;
};

----------------------------------
--[[	CUF NamePlate Options	]]
----------------------------------
local PersonalNamePlateFrameOptions=setmetatable({
--	colorHealthBySelection=false;
	displayAggroHighlight=true;
	displayName=false;
	displayStatusText=true;
	healthText="none";

--[[	DefaultCompactNamePlatePlayerFrameOptions from Modern WoW
	displaySelectionHighlight=false;
--	displayAggroHighlight=false;
	displayName=false;
	fadeOutOfRange=false;
	colorNameBySelection=true;
	smoothHealthUpdates=false;
--	displayNameWhenSelected=false;
	showLevel=true;

	healthBarColorOverride=CreateColor(0,1,0);
	defaultBorderColor=CreateColor(0,0,0,1);
--]]
},{__index=DefaultCompactNamePlateFriendlyFrameOptions});

----------------------------------
--[[	Personal NamePlate	]]
----------------------------------
local PersonalNamePlate_Initialized=false;

local PersonalNamePlate=CreateFrame("Frame");
PersonalNamePlate:SetPoint("CENTER",0,DefaultYPos);
PersonalNamePlate:SetSize(128,32);
PersonalNamePlate:Hide();

PersonalNamePlate:RegisterEvent("PLAYER_LOGIN");

PersonalNamePlate:RegisterEvent("PLAYER_ENTERING_WORLD");
PersonalNamePlate:RegisterEvent("PLAYER_REGEN_ENABLED");
PersonalNamePlate:RegisterEvent("PLAYER_REGEN_DISABLED");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXHEALTH","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH_FREQUENT","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_SHOW","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_HIDE","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_DISPLAYPOWER","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXPOWER","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_UPDATE","player");

PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_INTERRUPTED","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_DELAYED","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_START","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_UPDATE","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_STOP","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_START","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_STOP","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_FAILED","player");

local function PersonalNamePlate_UpdateVisibility(show)
	if not PersonalNamePlate_Initialized then return; end--	Don't run if we haven't loaded yet (ex. Options callback via ADDON_LOADED fires before PLAYER_LOGIN)

--	Visibility Logic
	local showbypower=false; do
		local powertype,powertoken=UnitPowerType("player");
		showbypower=UnitPower("player",powertype)~=(PowerRestState[powertype] or UnitPowerMax("player",powertype));
	end

	show=(	AddOn.Options.PersonalNamePlate_Enabled--	Enabled?
	and	UIParent:IsVisible()--	UI Shown?
	and	(
			AddOn.Options.PersonalNamePlate_AlwaysShow--	Always Show?
		or	(
				AddOn.OptionsPanel:IsVisible()--	Options Panel visible?
			or	(show and true)--	Force shown? (Cast to true; nil is already handled)
			or	(
					show==nil--	Omitted arg? (Preserves false value from above)
				and	(
						InCombatLockdown()--	Show in combat
					or	UnitHealth("player")<UnitHealthMax("player")--	Show if health missing
					or	(AddOn.Options.PowerBars_Enabled and showbypower)--	Show if power not at rest state (Only if PowerBars enabled)
					or	(
							AddOn.Options.PersonalNamePlate_EnableCastBar and not AddOn.IsVanillaClient--	CastBar option BCC+
						and	(UnitCastingInfo("player") or UnitChannelInfo("player"))--	Show if CastBar shown
						and	true or false--	Cast to boolean
						)
					)
				)
			)
		)
	);

--	Apply if changed
	if show~=PersonalNamePlate:IsShown() then
		PersonalNamePlate:SetShown(show);
	end
end

PersonalNamePlate:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_LOGIN" then--	PersonalNamePlate Init (allow addons hooking NamePlateDriverFrame:OnNamePlateCreated() to work on this)
		NamePlateDriverFrame:OnNamePlateCreated(self);--	Injects NamePlateBaseMixin
		do--	We want our UnitFrame, but asking the frame pool to create one for us spreads taint, so we hack the frame pool to create, but not store it
			self.UnitFrame=NamePlateDriverFrame.pools.pools.NamePlateUnitFrameTemplate:creationFunc();
			self.UnitFrame:SetParent(self);
			self.UnitFrame:SetPoint("TOPLEFT",self,"TOPLEFT");
			self.UnitFrame:EnableMouse(false);
			self:SetScript("OnSizeChanged",self.OnSizeChanged);
			self:OnSizeChanged();

			AddOn.OnNamePlateCreated(NamePlateDriverFrame,self)
--			NamePlateDriverFrame:AcquireUnitFrame(self);--	Original call for this (taints frame pool apparently)
		end
		self.UnitFrame.RaidTargetFrame:Hide();--		RaidTargetIcon is always hidden on player

--		Fix level text layering issue (Don't know why this bug doesn't happen on real NamePlates)
		self.UnitFrame.LevelFrame:SetFrameStrata(self.UnitFrame:GetFrameStrata());
		self.UnitFrame.LevelFrame:SetFrameLevel(self.UnitFrame.healthBar.border:GetFrameLevel()+1);

--		Apply Layout (NamePlateDriverFrame won't overwrite this since it never recieves NAME_PLATE_UNIT_ADDED for "player")
		CompactUnitFrame_SetUpFrame(PersonalNamePlate.UnitFrame,DefaultCompactNamePlateFriendlyFrameSetup);--	Expose creation as a friendly NamePlate
		CompactUnitFrame_SetOptionTable(PersonalNamePlate.UnitFrame,PersonalNamePlateFrameOptions);--	Set our own options table
		CompactUnitFrame_UpdateAll(PersonalNamePlate.UnitFrame);--	Update display (Called by CompactUnitFrame_SetUpFrame(), but not CompactUnitFrame_SetOptionTable())

--[[		Setup Conversion Notes
--		These are in NamePlateDriverFrame:ApplyFrameOptions()
		self:OnOptionsUpdated();--	Defunct in classic
		NamePlateDriverFrame:UpdateInsetsForType("friendly",self);--	This isn't a real NamePlate

		NamePlateDriverFrame:OnUnitAuraUpdate("player");
		NamePlateDriverFrame:OnRaidTargetUpdate();
--]]

--		Apply startup options
		self.UnitFrame.CastBar.showCastbar=AddOn.Options.PersonalNamePlate_EnableCastBar;

--		Safe to update visibility now
		PersonalNamePlate_Initialized=true;
	end

--	Force shown if PLAYER_REGEN_DISABLED is firing, otherwise use automated logic (InCombatLockdown() isn't true yet)
	PersonalNamePlate_UpdateVisibility(event=="PLAYER_REGEN_DISABLED" or nil);
end);

PersonalNamePlate:SetScript("OnShow",function(self)
	self:OnAdded("player",NamePlateDriverFrame);
	if not AddOn.IsVanillaClient then
		self.UnitFrame.CastBar.showTradeSkills=true;--	:OnAdded() calls CastingBarFrame_SetUnit() and sets this to false
	end
	NamePlateDriverFrame:OnUnitAuraUpdate("player");
end);
PersonalNamePlate:SetScript("OnHide",function(self) return self:OnRemoved(); end);

local function OptionsPanel_OnVisibilityChanged() return PersonalNamePlate_UpdateVisibility(); end
AddOn.OptionsPanel:HookScript("OnShow",OptionsPanel_OnVisibilityChanged);
AddOn.OptionsPanel:HookScript("OnHide",OptionsPanel_OnVisibilityChanged);
UIParent:HookScript("OnShow",OptionsPanel_OnVisibilityChanged);
UIParent:HookScript("OnHide",OptionsPanel_OnVisibilityChanged);

----------------------------------
--[[	Options Defaults	]]
----------------------------------
AddOn.Options.PersonalNamePlate_Enabled=true;
AddOn.Options.PersonalNamePlate_AlwaysShow=false;
AddOn.Options.PersonalNamePlate_EnableCastBar=true;
AddOn.Options.PersonalNamePlate_HideDefaultCastBar=true;
AddOn.Options.PersonalNamePlate_YPos=DefaultYPos;

--------------------------
--[[	Options Layout	]]
--------------------------
local SubPanel=AddOn.OptionsPanel:CreateSubPanel();
SubPanel:SetHeight(AddOn.IsVanillaClient and 104 or 152);

local EnableButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
EnableButton:SetPoint("TOPLEFT",4,0);
EnableButton.text:SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_Enabled);

local AlwaysShowButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
AlwaysShowButton:SetPoint("TOPLEFT",28,-24);
AlwaysShowButton.text:SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_AlwaysShow);

local EnableCastBarButton;
local HideDefaultCastBarButton;
if not AddOn.IsVanillaClient then
	EnableCastBarButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
	EnableCastBarButton:SetPoint("TOPLEFT",28,-48);
	EnableCastBarButton.text:SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_EnableCastBar);

	HideDefaultCastBarButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
	HideDefaultCastBarButton:SetPoint("TOPLEFT",52,-72);
	HideDefaultCastBarButton.text:SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_HideDefaultCastBar);
end

local YPosSlider=CreateFrame("Slider",nil,SubPanel,"OptionsSliderTemplate");
YPosSlider:SetPoint("TOP",0,AddOn.IsVanillaClient and -64 or -112);
YPosSlider:SetWidth(256);

YPosSlider:SetMinMaxValues(-384,384);
YPosSlider:SetValue(DefaultYPos);

YPosSlider:SetValueStep(16);
YPosSlider:SetStepsPerPage(1);
YPosSlider:SetObeyStepOnDrag(true);

YPosSlider.Mid=YPosSlider:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
YPosSlider.Mid:SetPoint("TOP",YPosSlider,"BOTTOM",0,3);

YPosSlider.Text:SetFontObject(GameFontHighlightSmall);
YPosSlider.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,DefaultYPos);
YPosSlider.Low:SetText(-384);
YPosSlider.Mid:SetText(0);
YPosSlider.High:SetText(384);

--------------------------
--[[	Options Setup	]]
--------------------------
AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_Enabled",EnableButton
	,nil
	,EnableButton.SetChecked
	,EnableButton.GetChecked
	,"OnClick"
);

AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_AlwaysShow",AlwaysShowButton
	,nil
	,AlwaysShowButton.SetChecked
	,AlwaysShowButton.GetChecked
	,"OnClick"
);

if not AddOn.IsVanillaClient then
	AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_EnableCastBar",EnableCastBarButton
		,nil
		,EnableCastBarButton.SetChecked
		,EnableCastBarButton.GetChecked
		,"OnClick"
	);

	AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_HideDefaultCastBar",HideDefaultCastBarButton
		,nil
		,HideDefaultCastBarButton.SetChecked
		,HideDefaultCastBarButton.GetChecked
		,"OnClick"
	);
end

AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_YPos",YPosSlider
	,nil
	,YPosSlider.SetValue
	,YPosSlider.GetValue
	,"OnValueChanged"
);

AddOn.OptionsPanel:HookOption("PersonalNamePlate_Enabled",function(_,_,val)
	local control_alwaysshow=AddOn.OptionsPanel:GetControl("PersonalNamePlate_AlwaysShow");
	control_alwaysshow:SetEnabled(val);
	control_alwaysshow.text:SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

	if not AddOn.IsVanillaClient then
		local control_enablecastbar=AddOn.OptionsPanel:GetControl("PersonalNamePlate_EnableCastBar");
		control_enablecastbar:SetEnabled(val);
		control_enablecastbar.text:SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

		do	local val=val and AddOn.Options.PersonalNamePlate_EnableCastBar;
			local control_hidedefaultcastbar=AddOn.OptionsPanel:GetControl("PersonalNamePlate_HideDefaultCastBar");
			control_hidedefaultcastbar:SetEnabled(val);
			control_hidedefaultcastbar.text:SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);
		end

		local show=not (val and AddOn.Options.PersonalNamePlate_EnableCastBar and AddOn.Options.PersonalNamePlate_HideDefaultCastBar);
		CastingBarFrame.showCastbar=show;
		if not show and CastingBarFrame:IsShown() then CastingBarFrame:Hide(); end
	end

	local control_ypos=AddOn.OptionsPanel:GetControl("PersonalNamePlate_YPos");
	control_ypos:SetEnabled(val);
	do	local fontobj=val and GameFontHighlightSmall or GameFontDisableSmall;
		control_ypos.Text:SetFontObject(fontobj);
		control_ypos.Low:SetFontObject(fontobj);
		control_ypos.Mid:SetFontObject(fontobj);
		control_ypos.High:SetFontObject(fontobj);
	end

	PersonalNamePlate_UpdateVisibility();
end);

AddOn.OptionsPanel:HookOption("PersonalNamePlate_AlwaysShow",function()
	PersonalNamePlate_UpdateVisibility();--	Directly setting this as the callback will cause it to recieve the control as the optional force show flag
end);

if not AddOn.IsVanillaClient then
	AddOn.OptionsPanel:HookOption("PersonalNamePlate_EnableCastBar",function(_,_,val)
		do	local val=val and AddOn.Options.PersonalNamePlate_Enabled;
			local control_hidedefaultcastbar=AddOn.OptionsPanel:GetControl("PersonalNamePlate_HideDefaultCastBar");
			control_hidedefaultcastbar:SetEnabled(val);
			control_hidedefaultcastbar.text:SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

		end

		local show=not (AddOn.Options.PersonalNamePlate_Enabled and val and AddOn.Options.PersonalNamePlate_HideDefaultCastBar);
		CastingBarFrame.showCastbar=show;
		if not show and CastingBarFrame:IsShown() then CastingBarFrame:Hide(); end

		if PersonalNamePlate_Initialized then
			local castbar=PersonalNamePlate.UnitFrame.CastBar;
			castbar.showCastbar=val;
			if not val and castbar:IsShown() then castbar:Hide(); end

			PersonalNamePlate_UpdateVisibility();
		end
	end);

	AddOn.OptionsPanel:HookOption("PersonalNamePlate_HideDefaultCastBar",function(_,_,val)
		local show=not (AddOn.Options.PersonalNamePlate_Enabled and AddOn.Options.PersonalNamePlate_EnableCastBar and val);
		CastingBarFrame.showCastbar=show;
		if not show and CastingBarFrame:IsShown() then CastingBarFrame:Hide(); end
	end);
end

AddOn.OptionsPanel:HookOption("PersonalNamePlate_YPos",function(self,_,val)
	self.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,val);
	PersonalNamePlate:SetPoint("CENTER",0,val);
end);

AddOn.OptionsPanel:HookOption("PowerBars_Enabled",function()
	if AddOn.Options.PersonalNamePlate_Enabled then PersonalNamePlate_UpdateVisibility(); end
end);
