--[[	PhantomPlates Utilities Library
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);
AddOn.Utilities={};

----------------------------------
--[[	Client Versioning	]]
----------------------------------
--[[	WOW_PROJECT_ID contains either of the following constants
	WOW_PROJECT_MAINLINE = 1		Modern WoW
	WOW_PROJECT_CLASSIC = 2			Classic Vanilla
	WOW_PROJECT_BURNING_CRUSADE_CLASSIC = 5	Classic Burning Crusade

	LE_EXPANSION_LEVEL_CURRENT contains either of the classic-only constants
	LE_EXPANSION_CLASSIC = 0
	LE_EXPANSION_BURNING_CRUSADE = 1
	LE_EXPANSION_NORTHREND = 2
--]]
AddOn.IsModernClient=(WOW_PROJECT_ID==WOW_PROJECT_MAINLINE);
AddOn.IsClassicClient=(WOW_PROJECT_ID~=WOW_PROJECT_MAINLINE);
AddOn.IsVanillaClient=(AddOn.IsClassicClient and LE_EXPANSION_LEVEL_CURRENT==LE_EXPANSION_CLASSIC);
AddOn.IsBCClient=(AddOn.IsClassicClient and LE_EXPANSION_LEVEL_CURRENT==LE_EXPANSION_BURNING_CRUSADE);
AddOn.IsWrathClient=(AddOn.IsClassicClient and LE_EXPANSION_LEVEL_CURRENT==LE_EXPANSION_NORTHREND);
--[[
AddOn.IsVanillaClient=(WOW_PROJECT_ID==WOW_PROJECT_CLASSIC);
AddOn.IsTBCClient=(WOW_PROJECT_ID==WOW_PROJECT_BURNING_CRUSADE_CLASSIC);
AddOn.InterfaceVersion=select(4,GetBuildInfo());
--]]

----------------------------------
--[[	Shared Functions	]]
----------------------------------
function AddOn.Utilities.AbsFloor(val) return math[val>=0 and "floor" or "ceil"](val); end
function AddOn.Utilities.AbsCeil(val) return math[val>=0 and "ceil" or "floor"](val); end

function AddOn.Utilities.ModifyFont(obj,font,size,flags)
	local origfont,origsize,origflags=obj:GetFont();
	return obj:SetFont(font or origfont,size or origsize,flags or origflags);
end

function AddOn.Utilities.UnitPowerBarColor(unit)
	local powertype,powertoken,r,g,b=UnitPowerType(unit);
	if PowerBarColor[powertoken] then return ColorMixin.GetRGB(PowerBarColor[powertoken]);
	elseif r and g and b then return r,g,b;
	else return ColorMixin.GetRGB(PowerBarColor[powertype] or PowerBarColor.MANA); end
end

function AddOn.Utilities.AdjustColorBrightness(delta,r,g,b)--	Delta is -1.0 to 1.0 (Negative darkens; Positive brightens)
	if delta>0 then
		return r+(1-r)*delta,g+(1-g)*delta,b+(1-b)*delta;
	else delta=1+delta; return r*delta,g*delta,b*delta; end
end

function AddOn.Utilities.ShouldKnowUnitHealth(unit)--	Lua implementation of how health obfuscation works in the current game
--	Note: Totems/Guardians/Etc are considered NPCs when it comes to health obfuscation
	local guid=UnitGUID(unit);
	local unittype=guid and guid:match("^(.-)%-");

	return	(unittype~="Player" and unittype~="Pet")--	Is NPC? (incl. totem/guardian/etc)
	or	(UnitIsUnit(unit,"player") or UnitIsUnit(unit,"pet"))--	Is player or player's pet?
	or	(UnitPlayerOrPetInRaid(unit) or UnitPlayerOrPetInParty(unit));--	Is in party/raid?
end
